public class VendingMachine {
  Item[] items;          // the items that can be bought
  double price;          // common price for all items
  double receivedAmount; // the amount of money entered into the machine
  double change;         // the amount of money the machine has left to give change
  double till;           // money that cannot be given back as change
  private Display display;  // display window where information is shown

  VendingMachine(Item[] items, double price, double change) {
    this.items = items;
    this.price = price;
    this.change = change;
    receivedAmount = 0.0;
    till = 0.0;
    display = new Display();
  }

  public void showMessage(String message) {
    display.showMessage(message);
  }

  public double findPrice(int itemSelected) {
    return price;
  }

  public void acceptCash(double amount) {
    receivedAmount += amount;
    showMessage("Accepted: " + formatMoney(receivedAmount) + " GBP");
  }

  public boolean itemAvailable(int selectedItem) {
    assert selectedItem >=0 && selectedItem < items.length :
      "Item number must be between 0 and " + (items.length-1);
    return (items[selectedItem].getInventory() > 0);
  }

  private void deliverItem(int selectedItem) {
    // here the vending machine will place chosen item in the receiver door
    showMessage("The item " + items[selectedItem].getItemName() + " is dispensed.");
  }

  private void giveChange(double itemChange) {
    // here the machine will give change.
    showMessage("Change " + formatMoney(itemChange) + " GBP given back.");
    change -= itemChange;
  }

  private void updateTill(double amount) {
    till += amount;
  }

  public void selectItem(int selectedItem) {
    assert selectedItem >=0 && selectedItem < items.length :
      "Item number must be between 0 and " + (items.length-1);
    if (itemAvailable(selectedItem)) {
      double price = findPrice(selectedItem);
      if (receivedAmount >= price) {
        deliverItem(selectedItem);
        items[selectedItem].setInventory(items[selectedItem].getInventory() - 1);
        giveChange(receivedAmount-price);
        updateTill(receivedAmount);
        receivedAmount = 0.0;
        showMessage("Thank you for buying our product - Welcome back!");
      }
      else { // received amount is less than its price
        showMessage("Not enough money inserted. " +
            items[selectedItem].getItemName() + " costs " + formatMoney(price) + " GBP");
      }
    }
    else { // out of stock
      showMessage("Sorry, but we are out of " + items[selectedItem].getItemName()
          + ". Money is returned.");
    }
  }

  public void cancel() {
    giveChange(receivedAmount);
    showMessage("Item purchase is cancelled.");
    receivedAmount = 0.0;
  }

  public void showSelections() {
    String str = "The vending machine contains these items:";
    str += this;
    str += "\nEach item costs: " + formatMoney(price) + " GBP";
    str += "\nChange available: " + formatMoney(change) + " GBP";
    str += "\nTill contains: " + formatMoney(till) + " GBP";
    str += "\nInserted amount: " + formatMoney(receivedAmount) + " GBP";
    System.out.println(str);
  }

  public String toString() { // overrides toString() from the Object class
    String str = "";
    for (int i=0; i<items.length; i++) {
      str += "\nItem number: " + i;
      str += items[i];
    }
    return str;
  }

  protected String formatMoney(double amount) {
    return String.format("%.2f", amount);
  }
}